<?php

namespace App\Http\Controllers;

use App\Data\ItemsCollection;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Log;
use Throwable;

class ItemController extends Controller
{
    /**
     * @var ItemsCollection
     */
    private ItemsCollection $itemsCollection;

    public function __construct(ItemsCollection $itemsCollection)
    {
        $this->itemsCollection = $itemsCollection;
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function search(Request $request): JsonResponse
    {
        $status = Response::HTTP_NO_CONTENT;
        $result = $this->itemsCollection->findItem($request->input('query'));
        $response = [];
        if ($result->isNotEmpty()) {
            $status = Response::HTTP_OK;
            $response = [
                'items' => $result->toArray(),
            ];
        }
        return response()->json($response, $status);
    }

    /**
     * @param string $id
     * @return JsonResponse
     */
    public function getItem(string $id): JsonResponse
    {
        try {
            return response()->json([
                'item' => $this->itemsCollection->getItemById($id),
            ]);
        } catch (Throwable $exception) {
            Log::error($exception->getMessage());
            Log::error($exception->getTraceAsString());
            return response()->json([
                'error' => 'Item not found.',
            ], Response::HTTP_NOT_FOUND);
        }
    }
}
